------------------------------------------------------------------------------------------------------
-- Declare the main addon variable
------------------------------------------------------------------------------------------------------
ProfessionBars = LibStub("AceAddon-3.0"):NewAddon("ProfessionBars", "AceEvent-3.0", "AceTimer-3.0")

------------------------------------------------------------------------------------------------------
-- Various local variables we need
------------------------------------------------------------------------------------------------------
local db
local jc_bar
local jc_movebutton
local insc_bar
local insc_movebutton

------------------------------------------------------------------------------------------------------
-- Option defaults
------------------------------------------------------------------------------------------------------
local defaults = {
	profile = {
		mode = 2,
		help = true,
		offsets = {
			jc = {
				x = 250,
				y = 250
			},
			insc = {
				x = 250,
				y = 250
			}
		}
	},
}

profbars_slashoptions = {
	name = "ProfessionBars",
	handler = ProfessionBars,
	type = 'group',
	args = {
		config = {
			type = 'input',
			name = 'config',
			desc = 'Open the Config dialog',
			set = "OpenConfig",
		},
	},
}

------------------------------------------------------------------------------------------------------
-- Table of all our needed frames
------------------------------------------------------------------------------------------------------
local profbars_frames = {
	-- Jewelcrafting frames
	profbars_jcbutton1	= {item = 23425,	type = "jc",	offsetx = 210,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_Adamantium"},
	profbars_jcbutton2	= {item = 36909,	type = "jc",	offsetx = 245,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_Cobalt"},
	profbars_jcbutton3	= {item = 36912,	type = "jc",	offsetx = 280,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_Saronite_01"},
	profbars_jcbutton4	= {item = 23424,	type = "jc",	offsetx = 175,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_FelIron"},
	profbars_jcbutton5	= {item = 10620,	type = "jc",	offsetx = 140,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_Thorium_02"},
	profbars_jcbutton6	= {item = 3858,		type = "jc",	offsetx = 105,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_Mithril_02"},
	profbars_jcbutton7	= {item = 2772,		type = "jc",	offsetx = 70,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_Iron_01"},
	profbars_jcbutton8	= {item = 2771,		type = "jc",	offsetx = 35,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_Tin_01"},
	profbars_jcbutton9	= {item = 2770,		type = "jc",	offsetx = 0,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_Copper_01"},
	
	-- Inscription frames
	profbars_inscbutton1	= {item = 39342,	type = "insc",	offsetx = 210,	offsety = 0,	texturepath = "Interface\\Icons\\inv_inscription_pigment_nether"},
	profbars_inscbutton2	= {item = 39343,	type = "insc",	offsetx = 245,	offsety = 0,	texturepath = "Interface\\Icons\\inv_inscription_pigment_azure"},
	--profbars_inscbutton3	= {item = 0,		type = "insc",	offsetx = 0,	offsety = 0,	texturepath = ""},
	profbars_inscbutton4	= {item = 39341,	type = "insc",	offsetx = 175,	offsety = 0,	texturepath = "Interface\\Icons\\inv_inscription_pigment_silvery"},
	profbars_inscbutton5	= {item = 39340,	type = "insc",	offsetx = 140,	offsety = 0,	texturepath = "Interface\\Icons\\inv_inscription_pigment_violet"},
	profbars_inscbutton6	= {item = 39339,	type = "insc",	offsetx = 105,	offsety = 0,	texturepath = "Interface\\Icons\\inv_inscription_pigment_emerald"},
	profbars_inscbutton7	= {item = 39338,	type = "insc",	offsetx = 70,	offsety = 0,	texturepath = "Interface\\Icons\\inv_inscription_pigment_golden"},
	profbars_inscbutton8	= {item = 39334,	type = "insc",	offsetx = 35,	offsety = 0,	texturepath = "Interface\\Icons\\inv_inscription_pigment_grey"},
	profbars_inscbutton9	= {item = 39151,	type = "insc",	offsetx = 0,	offsety = 0,	texturepath = "Interface\\Icons\\inv_inscription_pigment_white"}
}

------------------------------------------------------------------------------------------------------
-- Open the config dialog
------------------------------------------------------------------------------------------------------
function ProfessionBars:OpenConfig()
	InterfaceOptionsFrame_OpenToCategory("ProfessionBars")
end

------------------------------------------------------------------------------------------------------
-- Gets a player's level in a skill
------------------------------------------------------------------------------------------------------
function ProfessionBars:GetLevel(id)
	local spell = GetSpellInfo(id)
	for i=1, GetNumSkillLines() do
		local name, _, _, level = GetSkillLineInfo(i);
		if name == spell then
			return level
		end
	end
end

------------------------------------------------------------------------------------------------------
-- Searches a player's bags for an item id
------------------------------------------------------------------------------------------------------
local function SearchPlayerBags(itemID, occurance)
	--print("-------------");
	local found = 0;
	for i=0, 4 do
		local a = GetContainerNumSlots(i)
		for c=1, a do
			local link = GetContainerItemLink(i, c);
			local name = GetItemInfo(itemID);
			if link then
				if string.match(link, name) then
					found = found+1;
					--print("Occurance "..found..": Bag "..i..", Slot "..c..": Item is "..link..".");
					if found == occurance then
						--print("-------------");
						return i, c
					end
				end
			end
		end
	end
	--print("-------------");
	return nil
end

------------------------------------------------------------------------------------------------------
-- Add the help text to the various buttons
------------------------------------------------------------------------------------------------------
local function AddHelp(self, type)
	if not db.help then
		return;
	end

	self:AddLine(" ");
	if type == "jc" then
		self:AddLine("Left click to prospect the highest level ore");
		self:AddLine("Right click to prospect the lowest level ore");
	elseif type == "insc" then
		self:AddLine("Left click to mill the highest level herbs");
		self:AddLine("Right click to mill the lowest level herbs");
	end
end

------------------------------------------------------------------------------------------------------
-- What happens when the tooltip is shown
------------------------------------------------------------------------------------------------------
local function OnTooltipShow(self, frame)
	local a = frame:GetName();

	if not profbars_frames[a] then
		-- Frame didn't exist
		return
	end
	
	-- Add the normal item tooltip
	self:SetHyperlink("item:"..profbars_frames[a]["item"]..":0:0:0:0:0:0:0");
	
	-- Add the help text (won't add anything if the option says not to)
	AddHelp(self, profbars_frames[a]["type"]);
end

------------------------------------------------------------------------------------------------------
-- What happens when we mouseover a button
------------------------------------------------------------------------------------------------------
local function OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_NONE");
	GameTooltip:ClearAllPoints();
	GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT");
	GameTooltip:ClearLines()
	OnTooltipShow(GameTooltip, self)
	GameTooltip:Show()
end

------------------------------------------------------------------------------------------------------
-- What happens when the mouse leaves the button
------------------------------------------------------------------------------------------------------
local function OnLeave(self)
	GameTooltip:Hide();
end

------------------------------------------------------------------------------------------------------
-- Check if a player has a specific spell
------------------------------------------------------------------------------------------------------
local function PlayerHasSpell(spellID)
	-- We need only the spellname
	-- local spell = GetSpellInfo(spellID);

	-- Check if we have this in our spellbook
	return GetSpellInfo((select(1, GetSpellInfo(spellID))));
end

------------------------------------------------------------------------------------------------------
-- What happens when the "bar mode" changes
------------------------------------------------------------------------------------------------------
function ProfessionBars:ModeChange()
	-- Do any potential button changes
	ProfessionBars:ITEM_PUSH();

	if db.mode == 1 then
		-- We're now dislocating from tradeskill frame

		if PlayerHasSpell(25229) then
			-- Jewelcrafting
			jc_bar:ClearAllPoints();
			jc_bar:SetPoint("BOTTOMLEFT", UIParent, db.offsets.jc.x, db.offsets.jc.y);
			jc_movebutton:Show();
			jc_bar:Show();
		end

		if PlayerHasSpell(2259) then
			-- Alchemy
		end

		if PlayerHasSpell(2018) then
			-- Blacksmithing
		end

		if PlayerHasSpell(7411) then
			-- Enchanting
		end

		if PlayerHasSpell(4036) then	
			-- Engineering
		end

		if PlayerHasSpell(2366) then
			-- Herbalism
		end

		if PlayerHasSpell(45357) then
			-- Inscription
			insc_bar:ClearAllPoints();
			insc_bar:SetPoint("BOTTOMLEFT", UIParent, db.offsets.insc.x, db.offsets.insc.y);
			insc_movebutton:Show();
			insc_bar:Show();
		end
	else
		-- We're now attaching to tradeskill frame

		if PlayerHasSpell(25229) then
			profbars_jcmove:Hide();
			profbars_jewelcrafting:Hide();
			profbars_jewelcrafting:SetWidth(profbars_jewelcrafting:GetWidth() - 17.5);
		end

		if PlayerHasSpell(45357) then
			profbars_inscmove:Hide();
			profbars_inscription:Hide();
			profbars_inscription:SetWidth(profbars_inscription:GetWidth() - 17.5);
		end
	end
end

------------------------------------------------------------------------------------------------------
-- Function to grab the number of stacks of a certain item in our bags
------------------------------------------------------------------------------------------------------
local function GetNumStacks(itemID)
	local found = 0;
	for i = 0, 4 do
		local a =GetContainerNumSlots(i)
		for c = 1, a do
			local link = GetContainerItemLink(i, c);
			local name = GetItemInfo(itemID);
			if link and string.match(link, name) then
				found = found+1;
			end
		end
	end
	return found
end

------------------------------------------------------------------------------------------------------
-- Wrapper function for our options table
------------------------------------------------------------------------------------------------------
local function giveOptions()
	local options = {
		type = "group",
		name = "ProfessionBars",
		get = function(k) return db[k.arg] end,
		set = function(k, v) db[k.arg] = v; end,
		args = {
			mode = {
				name = "Mode",
				type = "select",
				width = "double",
				values = {
					[1] = "Separate (draggable) bar",
					[2] = "Attached to Tradeskill window",
				},
				arg = "mode",
				set = function(k, v) db.mode = v; ProfessionBars:ModeChange() end,
			},
			help = {
				type = "toggle",
				name = "Show Help Text",
				desc = "Turns help On/Off",
				arg = "help",
			},
		},
	}
	return options;
end

------------------------------------------------------------------------------------------------------
-- What happens when the addon is first initialised
------------------------------------------------------------------------------------------------------
function ProfessionBars:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("ProfBarsDB", defaults, "Default")
	db = self.db.profile

	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("ProfessionBars", giveOptions)
	local frame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ProfessionBars", "ProfessionBars")

	--LibStub("AceConfig-3.0"):RegisterChatCommand("profbars", function() InterfaceOptionsFrame_OpenToCategory(frame) end)

	LibStub("AceConfig-3.0"):RegisterOptionsTable("ProfessionBarsSlash", profbars_slashoptions, {"profbars", "professionbars"});
end

------------------------------------------------------------------------------------------------------
-- What happens when the addon is enabled
------------------------------------------------------------------------------------------------------
function ProfessionBars:OnEnable()
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("CHAT_MSG_SKILL");
	self:RegisterEvent("TRADE_SKILL_SHOW");
	self:RegisterEvent("TRADE_SKILL_CLOSE");
	self:RegisterEvent("ITEM_PUSH");
end

------------------------------------------------------------------------------------------------------
-- What happens when a button is clicked
------------------------------------------------------------------------------------------------------
local function ButtonClick(self, button, down)
	-- Fetch the name of the button
	local name = self:GetName();
	
	-- Store item we will be using
	local item;

	-- Store the spell we will be using
	local spell;
	
	-- Table for the used herbs
	local herbs;

	if (name == "profbars_inscbutton9") then
		-- Milling
		-- spell = 51005;
		-- if button = "RightButton" then
			-- if GetItemCount(765) >= 5 then
				-- item = 765;
			-- elseif GetItemCount(2447) >= 5 then
				-- item = 2447;
			-- elseif GetItemCount(2449) >= 5 then
				-- item = 2449;
			-- end
		-- else
			-- if GetItemCount(2449) >= 5 then
				-- item = 2449;
			-- elseif GetItemCoutn(2447) >= 5 then	
				-- item = 2447;
			-- elseif GetItemCount(765) >= 5 then
				-- item = 765;
			-- end
		-- end
		herbs = {765, 2447, 2449};
	elseif (name == "profbars_inscbutton8") then
		-- Milling
		-- spell = 51005;
		-- if button = "RightButton" then
			-- if GetItemCount(785) >= 5 then
				-- item = 785;
			-- elseif GetItemCount(2452) >= 5 then
				-- item = 2452;
			-- elseif GetItemCount(2450) >= 5 then
				-- item = 2450;
			-- elseif GetItemCount(3820) >= 5 then
				-- item = 3820;
			-- elseif GetItemCount(2453) >= 5 then
				-- item = 2453;
			-- end
		-- else
			-- if ProfessionBars:GetLevel(45357) < 25 and GetItemCount(785) >= 5 then
				-- item = 785;
			-- elseif GetItemCount(2453) >= 5 then
				-- item = 2453;
			-- elseif GetItemCount(3820) >= 5 then
				-- item = 3820;
			-- elseif GetItemCount(2450) >= 5 then
				-- item = 2450;
			-- elseif GetItemCount(2452) >= 5 then
				-- item = 2452;
			-- elseif GetItemCount(785) >= 5 then
				-- item = 785;
			-- end
		if (not (button == "RightButton")) and (ProfessionBars:GetLevel(45357) < 25) and (GetItemCount(785) >= 5) then
			spell = 51005;
			item = 785;
		else
			herbs = {785, 2452, 2450, 3820, 2453};
		end

	elseif (name == "profbars_inscbutton7") then
		herbs = {3355, 3369, 3356, 3357};

	elseif (name == "profbars_inscbutton6") then
		herbs = {3818, 3821, 3358, 3819};

	elseif (name == "profbars_inscbutton5") then
		herbs = {4625, 8831, 8836, 8838, 8839, 8846};
		
	elseif (name == "profbars_inscbutton4") then
		herbs = {13464, 13463, 13465, 13466, 13467};
	
	-- elseif (name == "profbars_inscbutton3") then
		-- Doesn't exist
		-- spell = 51005;


		
	elseif (name == "profbars_inscbutton2") then
		herbs = {36904, 37921, 36901, 36907, 39970, 39969, 36902, 36903, 36906, 36905};
		
	elseif (name == "profbars_inscbutton1") then
		herbs = {22789, 22786, 22785, 22787, 22790, 22791, 22793};

	elseif (name == "profbars_jcbutton9") then
		-- Prospecting
		spell = 31252;
		
		if GetItemCount(2770) >= 5 then
			item = 2770;
		end

	elseif (name == "profbars_jcbutton8") then
		-- Prospecting
		spell = 31252;
		
		if GetItemCount(2771) >= 5 then
			item = 2771;
		end

	elseif (name == "profbars_jcbutton7") then
		-- Prospecting
		spell = 31252;
		
		if GetItemCount(2772) >= 5 then
			item = 2772;
		end

	elseif (name == "profbars_jcbutton6") then
		-- Prospecting
		spell = 31252;
		
		if GetItemCount(3858) >= 5 then
			item = 3858;
		end

	elseif (name == "profbars_jcbutton5") then
		-- Prospecting
		spell = 31252;
		
		if GetItemCount(10620) >= 5 then
			item = 10620;
		end

	elseif (name == "profbars_jcbutton4") then
		-- Prospecting
		spell = 31252;
		
		if GetItemCount(23424) >= 5 then
			item = 23424;
		end

	elseif (name == "profbars_jcbutton3") then
		-- Prospecting
		spell = 31252;
		
		if GetItemCount(36912) >= 5 then
			item = 36912;
		end

	elseif (name == "profbars_jcbutton2") then
		-- Prospecting
		spell = 31252;
		
		if GetItemCount(36909) >= 5 then
			item = 36909;
		end

	elseif (name == "profbars_jcbutton1") then
		-- Prospecting
		spell = 31252;
		
		if GetItemCount(23425) >= 5 then
			item = 23425;
		end

	end
	if herbs then
		spell = 51005;
		if button == "RightButton" then
			for i=1, #herbs do
				if GetItemCount(herbs[i]) >= 5 then
					item = herbs[i];
					break;
				end
			end
		else
			i = #herbs;
			while i >= 1 do
				if GetItemCount(herbs[i]) >= 5 then
					item = herbs[i];
					break;
				end
				i = i-1;
			end
		end
	end
	if not item then
		-- We lacked source material
		if spell == 31252 then
			--Jewelcrafting
			print("You don't have enough ore to prospect.");
		elseif spell == 51005 then
			--Inscription
			print("You don't have enough herbs to mill.");
		end
		return;
	end
	
	-- Important variables
	local a, b = SearchPlayerBags(item, 1);
	local _, c = GetContainerItemInfo(a, b);
	local d, e, f

	if c < 5 then
		d, e = SearchPlayerBags(item, 2);
		_, f = GetContainerItemInfo(d, e);
		if (c + f) < 5 then
			SplitContainerItem(d, e, f);
		else
			SplitContainerItem(d, e, (5 - c));
		end
		PickupContainerItem(a, b);
		print("Re-stacked items, click again to start.");
	else
		self:SetAttribute("type", "spell");
		self:SetAttribute("spell", GetSpellInfo(spell));
		self:SetAttribute("target-item", GetItemInfo(item));
	end
end

------------------------------------------------------------------------------------------------------
-- Function for creating a new button
------------------------------------------------------------------------------------------------------
local function CreateButton(name, parent)
	-- Create the button frame
	local btn = CreateFrame("Button", name, parent, "SecureActionButtonTemplate");
	
	-- Set height
	btn:SetHeight(35);

	-- Set width
	btn:SetWidth(35);

	-- Enable it for clicks
	btn:SetScript("PreClick", ButtonClick);
	btn:RegisterForClicks("AnyUp");

	-- Position it
	btn:SetPoint("TOPLEFT", parent, "TOPLEFT", profbars_frames[name].offsetx, profbars_frames[name].offsety);

	-- Set textures
	btn:SetNormalTexture(profbars_frames[name].texturepath);
	btn:SetHighlightTexture(profbars_frames[name].texturepath, 1.5);

	-- Set mouseover/mouseout
	btn:SetScript("OnEnter", OnEnter);
	btn:SetScript("OnLeave", OnLeave);
end

------------------------------------------------------------------------------------------------------
-- Function that creates all the buttons that are needed
------------------------------------------------------------------------------------------------------
function ProfessionBars:PLAYER_ENTERING_WORLD()

	-- Check if we even have a profession
	local haveProfession = false;

	if PlayerHasSpell(2259) then
		-- Alchemy
		
		-- We have a profession
		--haveProfession = true;
	end

	if PlayerHasSpell(2018) then
		-- Blacksmithing
		--haveProfession = true;
	end

	if PlayerHasSpell(7411) then
		-- Enchanting
		
		-- We have a profession
		--haveProfession = true;
	end

	if PlayerHasSpell(4036) then	
		-- Engineering
		
		-- We have a profession
		--haveProfession = true;
	end

	if PlayerHasSpell(2366) then
		-- Herbalism
		
		-- We have a profession
		--haveProfession = true;
	end

	if PlayerHasSpell(45357) then
		-- Inscription

		-- Create the main frame
		insc_bar = CreateFrame("Frame", "profbars_inscription", UIParent);
		
		-- Set height
		insc_bar:SetHeight(35);

		-- Set movable
		insc_bar:SetMovable(true);	
		
		-- Parent it to UIParent and position where we saved it
		insc_bar:SetPoint("BOTTOMLEFT", UIParent, db.offsets.insc.x, db.offsets.insc.y);
		
		-- Set default width
		insc_bar:SetWidth(297.5);

		-- Create the Move button for the Inscription frame
		insc_movebutton = CreateFrame("Button", "profbars_inscmove", profbars_inscription);
		insc_movebutton:SetHeight(17.5);
		insc_movebutton:SetWidth(17.5);
		insc_movebutton:SetScript("OnMouseDown", profbars_startdragging_insc);
		insc_movebutton:SetScript("OnMouseUp", profbars_stopdragging_insc);
		insc_movebutton:SetPoint("TOPRIGHT", profbars_inscription, "TOPRIGHT");
		insc_movebutton:SetHighlightTexture("Interface\\Icons\\Spell_Arcane_PortalStormWind", 0.8);
		
		-- Button: Alabaster Pigment
		CreateButton("profbars_inscbutton9", profbars_inscription);
		
		-- Button: Dusky Pigment
		CreateButton("profbars_inscbutton8", profbars_inscription);
		
		-- Button: Golden Pigment
		CreateButton("profbars_inscbutton7", profbars_inscription);
		
		-- Button: Emerald Pigment
		CreateButton("profbars_inscbutton6", profbars_inscription);
		
		-- Button: Violet Pigment
		CreateButton("profbars_inscbutton5", profbars_inscription);
		
		-- Button: Silvery Pigment
		CreateButton("profbars_inscbutton4", profbars_inscription);
		
		-- stolenlegacy fails at numbering :p
		--CreateButton("profbars_inscbutton3", profbars_inscription);
		
		-- Button: Azure Pigment
		CreateButton("profbars_inscbutton2", profbars_inscription);
		
		-- Button: Nether Pigment
		CreateButton("profbars_inscbutton1", profbars_inscription);
		
		if db.mode == 2 then
			-- We're attaching to Tradeskill window
			if IsAddOnLoaded("AdvancedTradeSkillWindow") then
				insc_bar:ClearAllPoints();
				insc_bar:SetPoint("TOP", ATSWFrame, "BOTTOM", 0, 10);
			else
				insc_bar:ClearAllPoints();
				insc_bar:SetPoint("TOP", TradeSkillFrame, "BOTTOMLEFT", 180, 75);
			end

			-- We don't need the move button, or the main bar window
			insc_movebutton:Hide();
			insc_bar:Hide();
		end
		
		-- We have a profession
		haveProfession = true;
	end

	if PlayerHasSpell(25229) then
		-- Jewelcrafting
		
		-- Create the main frame
		jc_bar = CreateFrame("Frame", "profbars_jewelcrafting", UIParent);
		
		-- Set height
		jc_bar:SetHeight(35);
		
		-- Set movable
		jc_bar:SetMovable(true);
		
		-- Parent it to UIParent and position where we saved it
		jc_bar:SetPoint("BOTTOMLEFT", UIParent, db.offsets.jc.x, db.offsets.jc.y);
		
		-- Set default width
		jc_bar:SetWidth(332.5);
		
		-- Create the Move button for the JC frame
		jc_movebutton = CreateFrame("Button", "profbars_jcmove", profbars_jewelcrafting);
		jc_movebutton:SetHeight(17.5);
		jc_movebutton:SetWidth(17.5);
		jc_movebutton:SetScript("OnMouseDown", profbars_startdragging_jc);
		jc_movebutton:SetScript("OnMouseUp", profbars_stopdragging_jc);
		jc_movebutton:SetPoint("TOPRIGHT", profbars_jewelcrafting, "TOPRIGHT");
		jc_movebutton:SetHighlightTexture("Interface\\Icons\\Spell_Arcane_PortalStormWind", 0.8);
		
		-- Button: Copper Ore
		CreateButton("profbars_jcbutton9", profbars_jewelcrafting);
		
		-- Button: Tin Ore
		CreateButton("profbars_jcbutton8", profbars_jewelcrafting);
		
		-- Button: Iron Ore
		CreateButton("profbars_jcbutton7", profbars_jewelcrafting);
		
		-- Button: Mithril Ore
		CreateButton("profbars_jcbutton6", profbars_jewelcrafting);
		
		-- Button: Thorium Ore
		CreateButton("profbars_jcbutton5", profbars_jewelcrafting);
		
		-- Button: Fel Iron Ore
		CreateButton("profbars_jcbutton4", profbars_jewelcrafting);
		
		-- Button: Saronite Ore
		CreateButton("profbars_jcbutton3", profbars_jewelcrafting);
		
		-- Button: Cobalt Ore
		CreateButton("profbars_jcbutton2", profbars_jewelcrafting);
		
		-- Button: Adamantium Ore
		CreateButton("profbars_jcbutton1", profbars_jewelcrafting);

		if db.mode == 2 then
			-- We're attaching to Tradeskill window
			if IsAddOnLoaded("AdvancedTradeSkillWindow") then
				jc_bar:ClearAllPoints();
				jc_bar:SetPoint("TOP", ATSWFrame, "BOTTOM", 0, 10);
			else
				jc_bar:ClearAllPoints();
				jc_bar:SetPoint("TOP", TradeSkillFrame, "BOTTOMLEFT", 180, 75);
			end
			
			-- We don't need the move button, or the main bar window
			jc_movebutton:Hide();
			jc_bar:Hide();
		end
		
		-- We have a profession
		haveProfession = true;
	end

	if PlayerHasSpell(2108) then
		-- Leatherworking
		
		-- We have a profession
		--haveProfession = true;
	end

	if PlayerHasSpell(2575) then
		-- Mining
		
		-- We have a profession
		--haveProfession = true;
	end

	if PlayerHasSpell(8613) then
		-- Skinning
		
		-- We have a profession
		--haveProfession = true;
	end

	if PlayerHasSpell(3908) then
		-- Tailoring
		
		-- We have a profession
		--haveProfession = true;
	end
	
	if haveProfession then
		-- We have some profession, schedule a push of the items in 1 second
		ProfessionBars:ScheduleTimer("ITEM_PUSH", 1);
	end
end

------------------------------------------------------------------------------------------------------
-- What happens when we start dragging the JC frame
------------------------------------------------------------------------------------------------------
function profbars_startdragging_jc()
	profbars_jewelcrafting:StartMoving()
end

------------------------------------------------------------------------------------------------------
-- What happens when we stop dragging the JC frame
------------------------------------------------------------------------------------------------------
function profbars_stopdragging_jc()
	db.offsets.jc.x = profbars_jewelcrafting:GetLeft();
	db.offsets.jc.y = profbars_jewelcrafting:GetBottom();
	profbars_jewelcrafting:StopMovingOrSizing();
end

------------------------------------------------------------------------------------------------------
-- What happens when we start dragging the Inscription frame
------------------------------------------------------------------------------------------------------
function profbars_startdragging_insc()
	profbars_inscription:StartMoving()
end

------------------------------------------------------------------------------------------------------
-- What happens when we stop dragging the Inscription frame
------------------------------------------------------------------------------------------------------
function profbars_stopdragging_insc()
	db.offsets.insc.x = profbars_inscription:GetLeft();
	db.offsets.insc.y = profbars_inscription:GetBottom();
	profbars_inscription:StopMovingOrSizing();
end

------------------------------------------------------------------------------------------------------
-- We've gained a new skill level		UNTESTED!
------------------------------------------------------------------------------------------------------
function ProfessionBars:CHAT_MSG_SKILL(event, msg)
	if string.match(msg, (GetSpellInfo(25229))) or string.match(msg,GetSpellInfo(45357)) then
		ProfessionBars:ITEM_PUSH();
	end
end

------------------------------------------------------------------------------------------------------
-- What happens when we open tradeskill window
------------------------------------------------------------------------------------------------------
function ProfessionBars:TRADE_SKILL_SHOW()
	if db.mode == 1 then
		return;
	end

	local a = GetTradeSkillLine();
	if string.match(a, "%S+") == GetSpellInfo(25229) then
		if IsAddOnLoaded("AdvancedTradeSkillWindow") then
			profbars_jewelcrafting:ClearAllPoints();
			profbars_jewelcrafting:SetPoint("TOP", ATSWFrame, "BOTTOM", 0, 10);
		else
			profbars_jewelcrafting:ClearAllPoints();
			profbars_jewelcrafting:SetPoint("TOP", TradeSkillFrame, "BOTTOMLEFT", 180, 75);
		end
		if PlayerHasSpell(45357) then
			profbars_inscription:Hide()
		end
		profbars_jewelcrafting:Show();
	elseif string.match(a, "%S+") == GetSpellInfo(45357) then
		if IsAddOnLoaded("AdvancedTradeSkillWindow") then
			profbars_inscription:ClearAllPoints();
			profbars_inscription:SetPoint("TOP", ATSWFrame, "BOTTOM", 0, 10);
		else
			profbars_inscription:ClearAllPoints();
			profbars_inscription:SetPoint("TOP", TradeSkillFrame, "BOTTOMLEFT", 180, 75);
		end
		if PlayerHasSpell(25229) then
			profbars_jewelcrafting:Hide();
		end
		profbars_inscription:Show();
	else
		if PlayerHasSpell(25229) then
			profbars_jewelcrafting:Hide();
		end
		if PlayerHasSpell(45357) then
			profbars_inscription:Hide();
		end
	end
	ProfessionBars:ITEM_PUSH();
end

------------------------------------------------------------------------------------------------------
-- What happens when we close tradeskill window
------------------------------------------------------------------------------------------------------
function ProfessionBars:TRADE_SKILL_CLOSE()
	if db.mode == 1 then
		return;
	end

	if PlayerHasSpell(25229) then
		profbars_jewelcrafting:Hide();
	end
	if PlayerHasSpell(45357) then
		profbars_inscription:Hide();
	end
end

-------------------------------------------------------------------------------------------------------
-- Mecha-function for checking what buttons to show, and shiz; this is called whenever the player loots
-------------------------------------------------------------------------------------------------------
function ProfessionBars:ITEM_PUSH()
	if PlayerHasSpell(25229) then
		local c = ProfessionBars:GetLevel(25229);
		local a = {};
		if GetItemCount(2770) >= 5 and c >= 20 then
			local b = #a+1;
			a[b] = profbars_jcbutton9;
		end
		if GetItemCount(2771) >= 5 and c >= 50 then
			local b = #a + 1;
			a[b] = profbars_jcbutton8;
		end
		if GetItemCount(2772) >= 5 and c >= 125 then
			local b = #a+1;
			a[b] = profbars_jcbutton7;
		end
		if GetItemCount(3858) >= 5 and c >= 175 then
			local b = #a+1;
			a[b] = profbars_jcbutton6;
		end
		if GetItemCount(10620) >= 5 and c >= 250 then
			local b = #a+1;
			a[b] = profbars_jcbutton5;
		end
		if GetItemCount(23424) >= 5 and c >= 275 then
			local b = #a+1;
			a[b] = profbars_jcbutton4;
		end
		if GetItemCount(23425) >= 5 and c >= 325 then
			local b = #a+1;
			a[b] = profbars_jcbutton1;
		end
		if GetItemCount(36909) >= 5 and c >= 350 then
			local b = #a+1;
			a[b] = profbars_jcbutton2;
		end
		if GetItemCount(36912) >= 5 and c >= 400 then
			local b = #a+1;
			a[b] = profbars_jcbutton3;
		end

		if db.mode == 1 then
			profbars_jewelcrafting:SetWidth(#a * 35 + 17.5);
		else
			profbars_jewelcrafting:SetWidth(#a * 35);
		end

		profbars_jcbutton1:Hide();
		profbars_jcbutton2:Hide();
		profbars_jcbutton3:Hide();
		profbars_jcbutton4:Hide();
		profbars_jcbutton5:Hide();
		profbars_jcbutton6:Hide();
		profbars_jcbutton7:Hide();
		profbars_jcbutton8:Hide();
		profbars_jcbutton9:Hide();
		profbars_jcmove:Hide();

		for i = 1, #a do
			a[i]:ClearAllPoints()
			a[i]:SetPoint("TOPLEFT", profbars_jewelcrafting, "TOPLEFT", (i - 1) * 35, 0);
			a[i]:Show();
		end
		if db.mode == 1 and #a > 0 then
			profbars_jcmove:Show();
		end
	end

	if PlayerHasSpell(45357) then
		-- Check Inscription stuffs
		
		local c = ProfessionBars:GetLevel(45357);

		local a = {};
		if ((GetItemCount(2449) >= 5) or (GetItemCount(2447) >= 5) or (GetItemCount(765) >= 5)) and c >= 1 then
			local b = #a+1;
			a[b] = profbars_inscbutton9;
		end
		if (((GetItemCount(2450)>=5) or (GetItemCount(2453)>=5) or (GetItemCount(3820)>=5) or (GetItemCount(2452)>=5)) and c >= 25) or (GetItemCount(785) >= 5 and c >= 1) then
			local b = #a+1;
			a[b] = profbars_inscbutton8;
		end
		if ((GetItemCount(3369) >= 5) or (GetItemCount(3356) >= 5) or (GetItemCount(3357) >= 5) or (GetItemCount(3355)>=5)) and c >= 75 then
			local b = #a+1;
			a[b] = profbars_inscbutton7;
		end
		if ((GetItemCount(3818) >= 5) or (GetItemCount(3821) >= 5) or (GetItemCount(3358) >= 5) or (GetItemCount(3819)>=5)) and c >= 125 then
			local b = #a+1;
			a[b] = profbars_inscbutton6;
		end
		if ((GetItemCount(8836) >= 5) or (GetItemCount(8839) >= 5) or (GetItemCount(4625)>=5) or (GetItemCount(8845)>=5) or (GetItemCount(8846)>=5) or (GetItemCount(8831) >= 5) or (GetItemCount(8838)>=5)) and c >= 175 then
			local b = #a+1;
			a[b] = profbars_inscbutton5;
		end
		if ((GetItemCount(13463) >= 5) or (GetItemCount(13464) >= 5) or (GetItemCount(13467) >= 5) or (GetItemCount(13465) >= 5) or (GetItemCount(13466) >= 5)) and c >= 225 then
			local b = #a+1;
			a[b] = profbars_inscbutton4;
		end
		if ((GetItemCount(22790)>=5) or (GetItemCount(22786) >= 5) or (GetItemCount(22785) >= 5) or (GetItemCount(22793) >= 5) or (GetItemCount(22791) >= 5) or (GetItemCount(22792)>=5) or (GetItemCount(22787)>=5) or (GetItemCount(22789)>=5)) and c >= 275 then
			local b = #a+1;
			a[b] = profbars_inscbutton1;
		end
		if ((GetItemCount(36903)>=5) or (GetItemCount(37921)>=5) or (GetItemCount(39970) >= 5) or (GetItemCount(36901)>=5) or (GetItemCount(36906)>=5) or (GetItemCount(36905)>=5) or (GetItemCount(36907)>=5) or (GetItemCount(36904)>=5)) and c >= 325 then
			local b = #a+1;
			a[b] = profbars_inscbutton2;
		end

		if db.mode == 1 then
			profbars_inscription:SetWidth(#a * 35 + 17.5);
		else
			profbars_inscription:SetWidth(#a * 35);
		end

		profbars_inscbutton1:Hide();
		profbars_inscbutton2:Hide();
		profbars_inscbutton4:Hide();
		profbars_inscbutton5:Hide();
		profbars_inscbutton6:Hide();
		profbars_inscbutton7:Hide();
		profbars_inscbutton8:Hide();
		profbars_inscbutton9:Hide();

		for i = 1, #a do
			a[i]:ClearAllPoints();
			a[i]:SetPoint("TOPLEFT", profbars_inscription, "TOPLEFT", (i - 1) * 35, 0);
			a[i]:Show();
		end

		if db.mode == 1 and #a > 0 then
			profbars_inscmove:Show();
		end
	end
end